﻿// substr_count - used by ipv6 address validation function in order to find the number of ':' 
function substr_count (haystack, needle, offset, length) 
{    
	pos = 0, cnt = 0;
	haystack += '';
	needle += '';
	
	if (isNaN(offset))
	{
		offset = 0;
	}

	if (isNaN(length))
	{
		length = 0;
	}

	offset--;  

	while ((offset = haystack.indexOf(needle, offset+1)) != -1)
	{
		if (length > 0 && (offset+needle.length) > length)
		{
			return false; 
		}
		else
		{
			cnt++; 
		} 
	}  
	return cnt; 
}

// the function trims a string (remove spaced before and after a string)
function trim(stringToTrim)
{
	return stringToTrim.replace(/^\s+|\s+$/g,"");
}

// Check if a single char is between lower to upper chare range 
function isBound(charCheck, charLower, charUpper)
{
	if(charCheck >= charLower && charCheck <= charUpper)
		return 1;
		
	return 0;
}

// test_ipv4
//
// Test for a valid dotted IPv4 address
//
function test_ipv4(address_field_value, bAllowBlank)
{
	var ipaddr_splt;
	
	if((address_field_value.length == 0) && (bAllowBlank == true))
		return true;
	
	// split IPv4 parts by '.'
	ipaddr_splt = address_field_value.split("."); 
	if (ipaddr_splt.length !=4)
		return false;

	// check all part of IPv4
	for (var e in ipaddr_splt)
	{
		if ((isNaN(ipaddr_splt[e]))||(ipaddr_splt[e]<0)||(ipaddr_splt[e]>255)||(ipaddr_splt[e]=='')||(ipaddr_splt[e].indexOf(' ')!=-1))
			return false;
	}
	
	return true;
}

// ipv6_address_validation_filter 
// 
// Test if the input is a valid ipv6 address 
function ipv6_address_validation_filter(address_field) 
{
	// Test for empty address
	if (address_field.length<3)
	{
		return address_field == "::"; 
	}

	// Check if part is in IPv4 format 
	if (address_field.indexOf('.')>0)
	{
		lastcolon = address_field.lastIndexOf(':');
		if (!(lastcolon && test_ipv4(address_field.substr(lastcolon + 1), false)))
			return false;

		// replace IPv4 part with dummy 
		address_field = address_field.substr(0, lastcolon) + ':0:0'; 
	}

	// Check uncompressed 
	if (address_field.indexOf('::')<0)
	{
		var match = address_field.match(/^(?:[a-f0-9]{1,4}:){7}[a-f0-9]{1,4}$/i); 
		return match != null; 
	}

	// Check colon-count for compressed format 
	if (substr_count(address_field, ':'))
	{
		var match = address_field.match(/^(?::|(?:[a-f0-9]{1,4}:)+):(?:(?:[a-f0-9]{1,4}:)*[a-f0-9]{1,4})?$/i); 
		return match != null; 
	}

	// Not a valid IPv6 address 
	return false; 
}


// ipv6_address_validation -  Test if the input is a valid ipv6 address 
function IPv6_address_validation(address_field,bAllowBlank)
{
	if((address_field.value.length == 0) && (bAllowBlank == true))
		return 'OK';
		
	// first IP address filter
	var regExp="(::|(([a-fA-F0-9]{1,4}):){7}(([a-fA-F0-9]{1,4}))|(:(:([a-fA-F0-9]{1,4})){1,6})|((([a-fA-F0-9]{1,4}):){1,6}:)|((([a-fA-F0-9]{1,4}):)(:([a-fA-F0-9]{1,4})){1,6})|((([a-fA-F0-9]{1,4}):){2}(:([a-fA-F0-9]{1,4})){1,5})|((([a-fA-F0-9]{1,4}):){3}(:([a-fA-F0-9]{1,4})){1,4})|((([a-fA-F0-9]{1,4}):){4}(:([a-fA-F0-9]{1,4})){1,3})|((([a-fA-F0-9]{1,4}):){5}(:([a-fA-F0-9]{1,4})){1,2}))";

	if (address_field.value.search(regExp)==-1)
	{
		return 'Invalid IPv6 address. Please enter adress as 1234::6F:1';
	}

	// second IP address filter
	if (!(ipv6_address_validation_filter(address_field.value)))
	{
		return 'Invalid IPv6 address. Please enter adress as 1234::6F:1';
	}

	return 'OK';
}   


// validate IPv6 - IPv6 Prefix validation //
function IPv6_prefix_validation(prefix_field,bAllowBlank)
{   			
	if((prefix_field.value.length == 0) && (bAllowBlank == true))
		return 'OK';

	if ((isNaN(prefix_field.value))||(prefix_field.value<1)||(prefix_field.value>128)||(prefix_field.value=='')||(prefix_field.value.indexOf(' ')!=-1))
	{
		return 'Please enter an IPv6 Prefix number in the range of 1-128';
	}
	
	return 'OK';
}   



/*  IPv4 element Address Validation - compose from 4 separates fields of 0-255
	check valididation of each field and print it's error if exist.
	return value: true - all fields are valid otherwise return false
*/
function ipv4_element_address_validation(addr_element_field,headerMsg,bAllowBlank)
{					
	var e;	    
	var i;
	
	/* check whether all fields are empty */
	if(bAllowBlank == true)
	{
		var e1;
		var e2;
		var e3;
		var e4;
		
		e1 = eval(addr_element_field + '0');
		e2 = eval(addr_element_field + '1');
		e3 = eval(addr_element_field + '2');
		e4 = eval(addr_element_field + '3');						
		
		if( (e1.value.length == 0)&&
			(e2.value.length == 0)&&
			(e3.value.length == 0)&&
			(e4.value.length == 0))
			return true;			
	}		
					
	/* check all fields validation data */
	for (i=0;i<4;i++)
	{
		e = eval(addr_element_field+i);

		if ((isNaN(e.value))||(e.value<0)||(e.value>255)||(e.value=='')||(e.value.indexOf(' ')!=-1))
		{						
			alert (headerMsg + ' Error: Invalid IPv4 address. Please enter IPv4 address as 192.168.0.50');
			e.focus();
			return false;
		}
	}
	
	return true;	
}



// IPv4 Address Validation //
function ipv4_address_validation(address_field,bAllowBlank)
{
	if(test_ipv4(address_field.value, bAllowBlank))
		return 'OK';

	return 'Invalid IPv4 address. Please enter IPv4 address as 192.168.0.50';
}


// IPV4 Prefix Validation //	
function ipv4_prefix_validation(prefix_field,bAllowBlank)
{
	if((prefix_field.value.length == 0) && (bAllowBlank == true))
		return 'OK';
		
	if ((isNaN(prefix_field.value))||(prefix_field.value<1)||(prefix_field.value>32)||(prefix_field.value=='')||(prefix_field.value.indexOf(' ')!=-1))
	{
		return 'Please enter an IPv4 Prefix number in the range of 1-32';
	}

	return "OK";	
}




// IPv4 mask Validation //
function ipv4_mask_validation(address_field,bAllowBlank)
{
	var ipaddr_splt;
	
	if((address_field.value.length == 0) && (bAllowBlank == true))
		return 'OK';

	// split IPv4 parts by '.' //
	ipaddr_splt = address_field.value.split("."); 
	if (ipaddr_splt.length !=4)
	{
		return 'Invalid IPv4 mask. Please enter IPv4 mask as 255.255.255.0';
	}

	// check all part of IPv4 //
	for (var e in ipaddr_splt)
	{
		if ((isNaN(ipaddr_splt[e]))||(ipaddr_splt[e]<0)||(ipaddr_splt[e]>255)||(ipaddr_splt[e]=='')||(ipaddr_splt[e].indexOf(' ')!=-1))
		{
			return 'Invalid IPv4 mask. Please enter IPv4 mask as 255.255.255.0';
		}
	}
				
	var correct_range = {128:1,192:1,224:1,240:1,248:1,252:1,254:1,255:1,0:1};
			
	for (var i = 0; i <= 3; i ++) 
	{
		if (!(parseInt(ipaddr_splt[i],10) in correct_range)) 
		{
			return 'Mask is not valid';
		}
	}
	
	if ((ipaddr_splt[0] == 0) || (ipaddr_splt[0] != 255 && ipaddr_splt[1] != 0) || (ipaddr_splt[1] != 255 && ipaddr_splt[2] != 0) || (ipaddr_splt[2] != 255 && ipaddr_splt[3] != 0)) 
	{
		return 'Mask is not valid';  
	}
	
	return 'OK';
}

// validate_hostname check if host name valid name
function validate_hostname(address_field, bAllowBlank)
{
	if((address_field.value.length == 0) && (bAllowBlank == true))
		return 'OK';

	var match = address_field.value.match(/^(([a-zA-Z]|[a-zA-Z][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$/i); 
	if(match != null)
		return 'OK';
		
	return 'Please enter valid host name';		
}

// validates if the user meant to write an IPv4 address
function validate_user_address_IPv4(address_field)
{
	for (var i in address_field.value)
	{
		if( (!isBound(address_field.value[i], '0', '9')) && ('.' != address_field.value[i]) )
			return 0;
	}
	return 1;
}

// validates if the user meant to write an IPv6 address
function validate_user_address_IPv6(address_field)
{
	for (var i in address_field.value)
	{
		if( !isBound(address_field.value[i], '0', '9') && !isBound(address_field.value[i], 'A', 'F') && 
			!isBound(address_field.value[i], 'a', 'f') && (':' != address_field.value[i]))
			return 0;	
	}
	return 1;
}

// validate_address_IPv4_IPv6 check IP address
function validate_address_IPv4_IPv6(address_field, bAllowBlank)
{
	//IPv4 Address validation
	msg = ipv4_address_validation(address_field, bAllowBlank);
	if (msg == 'OK')
		return 'OK';

	//if we here - address is not IPV4. lets check IPV6 address validation
	msg = IPv6_address_validation(address_field, bAllowBlank);
	if (msg == 'OK')
		return 'OK';

	//if we here - address is not IPV4 or IPV4 address //
	//lets find hints for the type that the user mean to type //
	if (validate_user_address_IPv6(address_field))  // find ':' - ipv6 sign //
	{
		return 'Please enter valid IPV6 address';
	}
	else if (validate_user_address_IPv4(address_field))  // find '.' - ipv4 sign //
	{
		return 'Please enter valid IPV4 address';
	}
	else // no special char was found //
	{
		return 'Please enter valid IPV4 or IPV6 address';		
	}   						
}
// validate_address_IPv4_IPv6_HOSTNAME check IP address or a valid HOSTNAME
function validate_address_IPv4_IPv6_HOSTNAME(address_field, bAllowBlank)
{
	//HOSTNAME Address validation
	msg = validate_hostname(address_field, bAllowBlank);
	if (msg == 'OK')
		return 'OK';

	msg = validate_address_IPv4_IPv6(address_field, bAllowBlank);
	if (-1 == msg.indexOf("Please enter valid IPV4 or IPV6 address"))
		return msg;
		
	return 'Please enter valid IPV4 or IPV6 address or host name';		
}
// validate IPV4V6 address - check IP address depend on the expected IPV type: IPV4,IPV6 or IPV4V6 (both).
function validate_address(Type,address_field,bAllowBlank)
{
	var msg;
	
	address_field.value = trim(address_field.value);
	if (Type == 'IPV4')
	{
		//IPv4 Address validation //
		return ipv4_address_validation(address_field,bAllowBlank);
	}
	else if (Type == 'IPV6')
	{
		//IPv6 Address validation //
		return IPv6_address_validation(address_field,bAllowBlank);
	}
	else if (Type == 'IPV4V6')
	{
		return validate_address_IPv4_IPv6(address_field, bAllowBlank);
	}
	else if (Type == 'HOSTNAME')
	{
		return validate_hostname(address_field, bAllowBlank);
	}
	else if (Type == 'IPV4V6HOSTNAME')
	{
		return validate_address_IPv4_IPv6_HOSTNAME(address_field, bAllowBlank);
	}
	else
	{
		return 'Unknown IPV TYPE';
	}
	
	return 'OK';
}


// validate IPV4V6 prefix - check IP address depend on the expected IPV type: IPV4,IPV6 or IPV4V6 (both).
function validate_prefix(Type, prefix_field,bAllowBlank)
{
	var msg;
	
	if (Type == 'IPV4')
	{
		//IPv4 Prefix validation // 
		return ipv4_prefix_validation(prefix_field,bAllowBlank);
	}
	else if (Type == 'IPV6')
	{
		//IPv6 Prefix validation // 	
		return IPv6_prefix_validation(prefix_field,bAllowBlank);		
	}
	else if (Type == 'IPV4V6')
	{
		//IPv4 Prefix validation // 
		msg = ipv4_prefix_validation(prefix_field,bAllowBlank);
		if (msg == 'OK')
			return 'OK';

		//IPv6 Prefix validation //   
		msg = IPv6_prefix_validation(prefix_field,bAllowBlank);
		if (msg != 'OK')
			return 'Please enter valid IPV4 or IPV6 address';
	}
	else
	{
		return 'Unknown IPV TYPE';
	}
	
	return 'OK';
}



// validate IPV4V6 address and prefix - check IP address depend on the expected IPV type: IPV4,IPV6 or IPV4V6 (both).
function validate_IPVType(Type, address_field, prefix_field,bAllowBlank)
{
	var msg;
	
	if (Type == 'IPV4')
	{
		//IPv4 Address validation //
		msg = ipv4_address_validation(address_field,bAllowBlank);   
		if (msg != 'OK')
			return msg;

		//IPv4 Prefix validation //
		return ipv4_prefix_validation(prefix_field);

	}
	else if (Type == 'IPV6')
	{
		//IPv6 Address validation //
		msg = IPv6_address_validation(address_field,bAllowBlank);
		if (msg != 'OK')
			return msg;

		//IPv6 Prefix validation //
		return IPv6_prefix_validation(prefix_field);
	}
	else if (Type == 'IPV4V6')
	{   			
		//IPv4 Address validation //
		msg = ipv4_address_validation(address_field,bAllowBlank);    
		if (msg == 'OK')
		{ 
			//IPv4 Prefix validation //
			return ipv4_prefix_validation(prefix_field);
		}

		//if we here - address is not IPV4. lets check IPV6 address validation //
		msg = IPv6_address_validation(address_field,bAllowBlank);
		if (msg == 'OK')
		{
			//IPv6 Prefix validation //
			return IPv6_prefix_validation(prefix_field);
		}

		//if we here - address is not IPV4 or IPV4 address //
		//lets find hints for the type that the user mean to type //
		if (address_field.value.indexOf(":")!=-1)   	 // find ':' - ipv6 sign //
		{
			return 'Please enter valid IPV6 address';
		}
		else if (address_field.value.indexOf(".")!=-1)   // find '.' - ipv4 sign //
		{
			return 'Please enter valid IPV4 address';
		}
		else
		{
			return 'Please enter valid IPV4 or IPV6 address';		
		}   						
	}
	else
	{
		return 'Unknown IPV TYPE';
	}
	
	return 'OK';
}


function check_result(resultMsg, field, headerMsg)
{
	if(resultMsg != "OK")
	{
		alert (headerMsg + ' Error: ' + resultMsg);
		field.focus();
		return false;
	}		
	
	return true;
}

// validates string has no '{' or '}' or '[' or ']', and containes only valid ASCII char
function validate_ascii(str)
{
	for (var i in str)
	{
		if ('[' == str[i]  || ']' == str[i] || '{' == str[i] || '}' == str[i])
			return 'Parenthesis characters are not allowed';
	}

	// verify all string chars are in rane 0-255 (other chars are represented by several chars)
	if (escape(str) != unescape(str))
		return 'None ascii characters are not allowed';

	return 'OK';
}
